/* VERSION 3.11 */
/*  24.02.2016  */
/* STM8 Version */

#include "nrf24l01.h"           //main header
#include "HW.h"                 //board definitions

/* SELECT HEADER FOR DIFFERENT HW */
#ifdef STM8S103
  #include "STM8S__SPI.h"
#endif
#ifdef STM8L101
  #include "STM8L_SPI.h"
#endif

nRF_ERROR_TypeDef RF_Send_Cmd(unsigned char adrs, unsigned char cmd);
unsigned char 	RF_Read_Cmd(unsigned char adrs);
void RF_Send_Adrs (unsigned char adrs, unsigned char * data);
void RF_IRQ_CLEAR (unsigned char CMD);
void RF_Flush (unsigned char CMD);

#ifndef RF_CE_PIN
	#error nRF24L01 Chip Enable Pin undefined
#endif
#ifndef RF_CSN_PIN
	#error nRF24L01 Chip Select Pin undefined
#endif
#ifndef RF_IRQ_PIN
	#error nRF24L01 IRQ Pin undefined
#endif

/**
  * @brief  Initialise NRF24L01 
  * @param  RF_InitStruct: pointer to an RF_InitTypeDef structure that contains
  *   the configuration information for the specified parameters.
  * @retval STATUS REG content (0x0E by default, 0x00 or 0xFF are impossible values)
  */
nRF_ERROR_TypeDef RF_Init (RF_InitTypeDef* RF_InitStruct)
{	
    nRF_ERROR_TypeDef res =         RF_Send_Cmd(CONFIG_REG, 
                                    RF_InitStruct->RF_Config | 
                                    RF_InitStruct->RF_Power_State | 
                                    RF_InitStruct->RF_CRC_Mode |
                                    RF_InitStruct->RF_Mode);
    if (res == 0x00 || res == 0xFF) return nRF_ERROR_CHIP_NOT_RESPONDING;
    RF_Send_Cmd(EN_AA_REG,RF_InitStruct->RF_Pipe_Auto_Ack);
    RF_Send_Cmd(EN_RXADDR_REG,RF_InitStruct->RF_Enable_Pipe);
    RF_Send_Cmd(SETUP_AW_REG,RF_InitStruct->RF_Setup);
    RF_Send_Cmd(SETUP_RETR_REG,(RF_InitStruct->RF_Auto_Retransmit_Count & 0x0F)|((RF_InitStruct->RF_Auto_Retransmit_Delay & 0x0F) << 4));
    RF_Send_Cmd(RF_CH_REG,(RF_InitStruct->RF_Channel & 0x7F));
    RF_Send_Cmd(RF_SETUP_REG,(0x01 | RF_InitStruct->RF_TX_Power | RF_InitStruct->RF_Data_Rate));
    RF_Send_Adrs(RX_ADDR_P0_REG,RF_InitStruct->RF_RX_Adress_Pipe0);
    RF_Send_Adrs(RX_ADDR_P1_REG,RF_InitStruct->RF_RX_Adress_Pipe1);
    RF_Send_Cmd(RX_ADDR_P2_REG,RF_InitStruct->RF_RX_Adress_Pipe2);
    RF_Send_Cmd(RX_ADDR_P3_REG,RF_InitStruct->RF_RX_Adress_Pipe3);
    RF_Send_Cmd(RX_ADDR_P4_REG,RF_InitStruct->RF_RX_Adress_Pipe4);
    RF_Send_Cmd(RX_ADDR_P5_REG,RF_InitStruct->RF_RX_Adress_Pipe5);
    RF_Send_Adrs(TX_ADDR_REG,RF_InitStruct->RF_TX_Adress);
    RF_Send_Cmd(RX_PW_P0_REG,RF_InitStruct->RF_Payload_Size_Pipe0);
    RF_Send_Cmd(RX_PW_P1_REG,RF_InitStruct->RF_Payload_Size_Pipe1);
    RF_Send_Cmd(RX_PW_P2_REG,RF_InitStruct->RF_Payload_Size_Pipe2);
    RF_Send_Cmd(RX_PW_P3_REG,RF_InitStruct->RF_Payload_Size_Pipe3);
    RF_Send_Cmd(RX_PW_P4_REG,RF_InitStruct->RF_Payload_Size_Pipe4);
    RF_Send_Cmd(RX_PW_P5_REG,RF_InitStruct->RF_Payload_Size_Pipe5);
    PIN_ON(RF_CE_PIN);
    return res;			 
}

/**
  * @brief  Send Payload via NRF24L01 
  * @param  unsigned char * data: pointer to first data memory location
  * @param  unsigned char DataLen: lenth of data array [1..32] for NRF24L01
  * @retval nRF_SUCCESS or nRF_ERROR_CHIP_NOT_RESPONDING if data send time overlimit
  */
nRF_ERROR_TypeDef RF_SendPayload (unsigned char * data, unsigned char DataLen)
{
  unsigned long DS_Delay = Payload_send_delay;
  SPI_Select(RF_CSN_PIN); 
  SPI_SendByte(RF_SendPayload_CMD);
  while(DataLen) 
  {
    SPI_SendByte(*data++);
    DataLen--;
  }
  SPI_Deselect(RF_CSN_PIN);
  PIN_ON(RF_CE_PIN);
  while(DS_Delay && PIN_SYG(RF_IRQ_PIN)) DS_Delay--;
  PIN_OFF(RF_CE_PIN);
  RF_IRQ_CLEAR(RF_TX_DS_IRQ_CLEAR);
  RF_Flush(RF_Flush_TX_CMD);
  if(DS_Delay == 0) return nRF_ERROR_CHIP_NOT_RESPONDING;
  return nRF_SUCCESS;
}

/**
  * @brief  Send Payload via NRF24L01 with Acknowledge
  * @param  unsigned char * data: pointer to first data memory location
  * @param  unsigned char DataLen: lenth of data array [1..32] for NRF24L01
  * @retval nRF_DATA_SEND_NO_ACK_RECEIVED if data send ok but no Acknowledge received 
            nRF_DATA_SEND_ACK_RECEIVED_OK if data send ok and Acknowledge received
            nRF_ERROR_CHIP_NOT_RESPONDING if data send time overlimit
  */
nRF_ERROR_TypeDef RF_SendPayloadACK (unsigned char * data, unsigned char DataLen) //return 0 if no ACK received, return 0xFF if ERROR
{
  unsigned long DS_Delay = Payload_send_delay;
  nRF_ERROR_TypeDef res = nRF_DATA_SEND_NO_ACK_RECEIVED;
  SPI_Select(RF_CSN_PIN); 
  SPI_SendByte(RF_SendPayload_CMD);
  while(DataLen) 
  {
    SPI_SendByte (*data++);
    DataLen--;
  }
  SPI_Deselect(RF_CSN_PIN);
  PIN_ON(RF_CE_PIN);
  while(DS_Delay && PIN_SYG(RF_IRQ_PIN)) DS_Delay--;
  PIN_OFF(RF_CE_PIN);

  SPI_Select(RF_CSN_PIN);
  if (SPI_ReadByte(0xFF) & RF_TX_DS_IRQ_CLEAR) 
    res = nRF_DATA_SEND_ACK_RECEIVED_OK; //IRQ reflected, ACK received
  SPI_Deselect(RF_CSN_PIN);
  
  if(DS_Delay == 0) res = nRF_ERROR_CHIP_NOT_RESPONDING;  
  RF_IRQ_CLEAR(RF_MAX_RT_IRQ_CLEAR | RF_TX_DS_IRQ_CLEAR);
  RF_Flush(RF_Flush_TX_CMD);

  return res; 
}

/**
  * @brief  Receive data from NRF24L01
  * @param  unsigned char * data: pointer to first data memory location
  * @param  unsigned char DataLen: lenth of data array [1..32] for NRF24L01
  * @retval unsigned char PipeNum - PIPE number of data received
    nRF_NO_DATA_RECEIVED if no data in FIFO and IRQ pin not reflected
  */
nRF_ERROR_TypeDef RF_Receive_Data (char * Data, unsigned char Data_Len)
{
	nRF_ERROR_TypeDef res;
	if (PIN_SYG(RF_IRQ_PIN)) return nRF_NO_DATA_RECEIVED;
	else
	{
		SPI_Select(RF_CSN_PIN);
		res = (nRF_ERROR_TypeDef)((SPI_ReadByte(0x61) & 0x0E)>>1);
		while(Data_Len)
		{
			*Data++ = SPI_ReadByte(0xFF);
			Data_Len--;
		}
		SPI_Deselect(RF_CSN_PIN);		
		
		if (RF_Read_Cmd(FIFO_STATUS_REG) & RF_RX_FIFO_EMPTY_Bit)	//check available data in RX FIFO
			RF_IRQ_CLEAR(RF_RX_DR_IRQ_CLEAR);	//if NO DATA in RX FIFO, clear IRQ
		return res;
	}			
}

/* INTERNAL FUNCTIONS DEFINITIONS */
/* NOTHING INTERESTING HERE */

nRF_ERROR_TypeDef RF_Send_Cmd (unsigned char adrs, unsigned char cmd)							//write data to register procedure
{	
	nRF_ERROR_TypeDef res;
	SPI_Select(RF_CSN_PIN);
	res = (nRF_ERROR_TypeDef) SPI_ReadByte((0x1F & adrs) | (1<<5));
	SPI_SendByte(cmd);
	SPI_Deselect(RF_CSN_PIN);			 
	return res;
}
unsigned char RF_Read_Cmd (unsigned char adrs)										//read data from register
{	
	unsigned char res;	
	SPI_Select(RF_CSN_PIN);
	SPI_SendByte(0x1F & adrs);
	res = SPI_ReadByte(0xFF);
	SPI_Deselect(RF_CSN_PIN);
	return res;
}

void RF_Send_Adrs (unsigned char adrs, unsigned char * data)
{	
  unsigned char cnt;
  SPI_Select(RF_CSN_PIN);
  SPI_SendByte(adrs | (1<<5));
  for (cnt = 0; cnt != Max_Adress_Len; cnt++) 
    SPI_SendByte(data[cnt]);	
  SPI_Deselect(RF_CSN_PIN); 
}

unsigned char RF_Carrier_Detect (void)										//returns 1 if Carrier Detected on current channel
{
	return (RF_Read_Cmd(CD_REG) & 0x01);
}

unsigned char RF_Count_Lost_Packets (void)									//returns num of Lost Packatets 
{
	return (((RF_Read_Cmd(OBSERV_TX_REG)) & 0xF0) >> 4);
}

unsigned char RF_Count_Resend_Packets (void)								//returns nut of Resend Packets
{
	return ((RF_Read_Cmd(OBSERV_TX_REG)) & 0x0F);
}

void RF_IRQ_CLEAR (unsigned char CMD)											//Clears IRQ
{
	RF_Send_Cmd(STATUS_REG, CMD);
} 

void RF_Flush (unsigned char CMD)
{
  SPI_Select(RF_CSN_PIN);
  SPI_SendByte(CMD);
  SPI_Deselect(RF_CSN_PIN);;
}

unsigned char RF_Mode_Set (unsigned char Mode)
{
	unsigned char pMode = RF_Read_Cmd(CONFIG_REG);
	RF_Send_Cmd(CONFIG_REG, (pMode & 0xFE) | (Mode & 1));
	return pMode;
}

nRF_ERROR_TypeDef RF_Sleep (void)
{
 return RF_Send_Cmd(CONFIG_REG, RF_Read_Cmd(CONFIG_REG) & ~RF_Power_On);  
}

nRF_ERROR_TypeDef RF_WakeUp (void)
{
 return RF_Send_Cmd(CONFIG_REG, RF_Read_Cmd(CONFIG_REG) | RF_Power_On); 
}

nRF_ERROR_TypeDef RF_FastTX_Address_Confirure (char * Address)
{
  unsigned char cnt; 
  nRF_ERROR_TypeDef res;
  SPI_Select(RF_CSN_PIN);
  res = (nRF_ERROR_TypeDef)SPI_ReadByte(TX_ADDR_REG | (1<<5));
  for (cnt = 0; cnt != 5; cnt++)
    SPI_SendByte(Address[cnt]);	
  SPI_Deselect(RF_CSN_PIN);
  return res;
}